import time
import os
import math
import random
import shutil

class CplexBenchmarkGen:
	def __init__(self):
		self.variableCounter = 0;
		self.constraintCounter = 0;
		self.allBinaryVars = []
	
	def getCost(self,i,j):
		return self.costMatrix[i][j];
	
	def getaFileobject(self, outputFilePath):
		filename = outputFilePath
		if os.path.exists(filename):
			os.remove(filename)
		fo = open(filename, "a")
		return fo
	
	def getNewVar(self):
		self.variableCounter += 1
		newvar = "x" + str(self.variableCounter)
		self.allBinaryVars.append(newvar)
		return newvar
	
	def getEdgekey(self, i, j, k):
		return "edge({0},{1},{2})".format(i,j,k)
		
	def objectiveFunc(self, nodes):
		s = "min: \n";		
		self.edge_variable_map = {}		
		edgeVarMapFilePath = self.setFolderPath + "/" + "EdgeVarMapFile_TDTSP_FGG_Enc.map";
		if(os.path.exists(edgeVarMapFilePath)):
			os.remove(edgeVarMapFilePath)		
		edgeVarMapFile = self.getaFileobject(edgeVarMapFilePath)
		
		for i in nodes:
			for j in nodes:
				if(i == j):
					continue
				for k in nodes:
					edgeKey = self.getEdgekey(i,j,k)
					edgeVar = self.getNewVar()
					self.edge_variable_map[edgeKey] = edgeVar
					cost = self.getCost(i-1,j-1)
					s += "+{0} {1} ".format(cost,edgeVar)					
					evmStr = edgeKey + " : " + self.edge_variable_map[edgeKey] + "\n"
					edgeVarMapFile.write(evmStr)				
					self.allBinaryVars.append(edgeVar)				
				s +="\n"
		return s[:-1] + ";"
	
	def inOutDegreeConstraints(self, nodes):
		#print len(nodes)
		s = ""
		for i in nodes:
			sumStr = ""	
			for j in nodes:
				if(i == j):
					continue				
				for k in nodes:
					edgeKey = self.getEdgekey(i,j,k)
					edgeVar = self.edge_variable_map[edgeKey]
					sumStr += "+1 {0} ".format(edgeVar)
				# for formatiing
				if(j == len(nodes)):
					sumStr = sumStr
				elif(i == len(nodes) and j == len(nodes) - 1):
					sumStr = sumStr
				else:
					sumStr += "\n"
										
			sumStr = sumStr + " = 1 ;\n\n"
			self.constraintCounter += 1
			s += sumStr	
		
		s += "\n"
		for j in nodes:
			sumStr = ""			
			for i in nodes:
				if(i == j):
					continue
				for k in nodes:
					edgeKey = self.getEdgekey(i,j,k)
					edgeVar = self.edge_variable_map[edgeKey]
					sumStr += "+1 {0} ".format(edgeVar)
				# for formatiing
				if(i == len(nodes)):
					sumStr = sumStr
				elif(j == len(nodes) and i == len(nodes) - 1):
					sumStr = sumStr
				else:
					sumStr += "\n"										
			sumStr = sumStr + " = 1 ;\n\n"
			self.constraintCounter += 1
			s += sumStr	
		
		return s
		
	def subToursEliminationConstraints(self, nodes):
		s ="\n"
		totalNodes = len(nodes)
		#Eq 32
		for i in range(2, totalNodes + 1):
			sum_ij_Str = ""
			sum_ji_Str = ""
			for j in range(1, totalNodes + 1):
				if(i == j ): continue
				
				for k in range(2, totalNodes + 1):
					edgeKey = self.getEdgekey(i,j,k)
					r_ijkVar = self.edge_variable_map[edgeKey]
					sum_ij_Str += "+{1} {0} ".format(r_ijkVar, k)
				sum_ij_Str += "\n"
				
				for k in range(1, totalNodes + 1):
					edgeKey = self.getEdgekey(j,i,k)
					r_jik = self.edge_variable_map[edgeKey]
					sum_ji_Str += "-{1} {0} ".format(r_jik, k)
				
				if(j == totalNodes):
					sum_ji_Str = sum_ji_Str
				elif(i == totalNodes and j == totalNodes - 1):
					sum_ji_Str = sum_ji_Str
				else:
					sum_ji_Str += "\n"
				
			s += sum_ij_Str
			s += sum_ji_Str + " = 1 ;"
			self.constraintCounter += 1
			s += "\n\n"
		#print s
		return s
		
	
	def getBounds(self, nodes):
		s ="Bounds\n"
		totalNodes = len(nodes)
		for i in range(2, totalNodes + 1):
			c = "2 <= u{0} <= {1}\n".format(i, totalNodes)
			s += c
		s += "\n"
		return s
	
	def getBinaries(self, nodes):
		s ="Binary\n"
		totalNodes = len(nodes)
		for binaryVar in self.allBinaryVars:
			s += binaryVar + "\n"
		return s	
		
	def generateEncoding(self,totalNodes):
		nodes = []
		self.allBinaryVars = []
		self.variableCounter = 0;
		self.constraintCounter = 0;
		
		#print nodes
		for n in range(1, totalNodes + 1):
			nodes.append(n);
		#print nodes
		PBstring = "";
		# Eq 28
		PBstring += self.objectiveFunc(nodes) + "\n\n"
		# Eq 29
		PBstring += self.inOutDegreeConstraints(nodes)
		# Eq 32
		PBstring += self.subToursEliminationConstraints(nodes)		
		fileHeader = "* #variable= {0} #constraint= {1}\n".format(self.variableCounter, self.constraintCounter)
		PBstring = fileHeader + PBstring
		PBstring = PBstring
		
		return PBstring
	
	def printCostMatrix(self, costMatrix):
		for i in range(len(costMatrix)):
			print costMatrix[i]
	
	def getCostMatrixFromFile(self, costMatrixFilePath):
		costMatrix = []
		self.numberOfNodes = 0;
		self.euclideanMaximumCost = 0;
		with open(costMatrixFilePath) as infile:
			for line in infile:			
				costs = line.split(',')
				if(len(costs) == 1):
					self.numberOfNodes = int(costs[0])
					continue
				intCosts = []
				for c in costs:
					cost = int(c)
					intCosts.append(int(c))
					if( cost > self.euclideanMaximumCost):
						self.euclideanMaximumCost = cost						
				costMatrix.append(intCosts)
				
		return costMatrix
	
	def deleteThanCreateFolder(self, path):
		if(os.path.exists(path)):
			shutil.rmtree(path)
		os.makedirs(path)
	
	def createIfNotExists(self, path):
		if(os.path.exists(path) == False):
			os.makedirs(path)
	
	def generate(self):
		
		self.benchMarkFolder = "../../../BenchMarks";
		costFilesPath = ['EuclideanCost.txt','OceanDataCost.txt','RandomCost.txt',
				'noisy_EuclideanCost/RandomSymmetric_EuclideanCostModified.txt',
				'noisy_EuclideanCost/AsymmetricRandomNoisy_50PercentEuclideanCostModified.txt',
				'noisy_EuclideanCost/AsymmetricNoisy_50PercentEuclideanCostModified.txt',
				'noisy_EuclideanCost/SymmetricNoisy_50PercentEuclideanCostModified.txt'
				]
		
		#costFilesPath = ['OceanDataCost.txt']
		
		nodeFolders  = os.listdir(self.benchMarkFolder)
		for nodeFolderName in nodeFolders:
			print "-------------"
			#if(nodeFolderName != "5_Nodes" and nodeFolderName != "4_Nodes" and nodeFolderName != "6_Nodes"): continue;
			craftedFolderPath = self.benchMarkFolder + "/" + nodeFolderName
			if(os.path.isdir(craftedFolderPath)):
				setFolders  = os.listdir(craftedFolderPath)
				for setFolder in setFolders:
					if(setFolder != "baseTest_set"): continue;
					setFolderPath = craftedFolderPath + "/" + setFolder
					self.setFolderPath = setFolderPath
					
					encodingFolderPath = setFolderPath + "/encodings"
					self.createIfNotExists(encodingFolderPath)
					
					pboEncodingFolderPath = setFolderPath + "/encodings/sat4j_scip"
					self.createIfNotExists(pboEncodingFolderPath)
										
					cplexDLencodingFolderPath = setFolderPath + "/encodings/sat4j_scip/fggEncoding"		
					self.deleteThanCreateFolder(cplexDLencodingFolderPath)					
					
					#if(setFolder != "baseTest_set"): continue;
					
					costFilespath = setFolderPath + "/costFiles/";
					for costfile in costFilesPath:
						costFilePath = costFilespath + costfile
						if(os.path.exists(costFilePath)):
							self.costMatrix = self.getCostMatrixFromFile(costFilePath)
							#print costFilePath
							#self.printCostMatrix(self.costMatrix)
							encoding = self.generateEncoding(self.numberOfNodes)
							costFilename =  os.path.basename(costFilePath).split('.')[0]
							outputFilePath = cplexDLencodingFolderPath + "/opb_fgg_" + costFilename + ".opb"
							#print outputFilePath
							fileObj = open(outputFilePath,"w+")
							fileObj.write(encoding)
							fileObj.close()
						else:
							print "Not Exist: {0}".format(costFilePath)
				print nodeFolderName
				print "-------------"

if __name__ == '__main__':	
	gen = CplexBenchmarkGen()
	gen.generate()
